--- asterisk-1.0.cvs/asterisk/channels/chan_zap.c	2004-10-16 23:01:34.000000000 -0700
+++ asterisk-1.0.scrd/asterisk/channels/chan_zap.c	2004-10-19 03:54:02.000000000 -0700
@@ -380,7 +380,7 @@
 struct zt_pri;
 #endif
 
-#define SUB_REAL		0			/* Active call */
+#define SUB_REAL	0			/* Active call */
 #define SUB_CALLWAIT	1			/* Call-Waiting call on hold */
 #define SUB_THREEWAY	2			/* Three-way call */
 
@@ -8538,6 +8538,7 @@
 			ast_cli(fd, "Default law: %s\n", tmp->law == ZT_LAW_MULAW ? "ulaw" : tmp->law == ZT_LAW_ALAW ? "alaw" : "unknown");
 			ast_cli(fd, "Fax Handled: %s\n", tmp->faxhandled ? "yes" : "no");
 			ast_cli(fd, "Pulse phone: %s\n", tmp->pulsedial ? "yes" : "no");
+			ast_cli(fd, "Gains Rx: %2.1f Tx: %2.1f\n", tmp->rxgain, tmp->txgain); 
 			ast_cli(fd, "Echo Cancellation: %d taps%s, currently %s\n", tmp->echocancel, tmp->echocanbridged ? "" : " unless TDM bridged", tmp->echocanon ? "ON" : "OFF");
 			if (tmp->master)
 				ast_cli(fd, "Master Channel: %d\n", tmp->master->channel);
@@ -8635,6 +8636,66 @@
 	handle_zap_show_cadences, "List cadences",
 	zap_show_cadences_help, NULL };
 
+static char zap_set_gain_usage[] =
+	"Usage: zap set gain <chan num> <Rx db> <Tx db>\n"
+	"	Immediatly override the gains that were loaded from zapata.conf\n"
+	"	This also affects in progress calls - use with care\n";
+
+static int handle_zap_set_gain(int fd, int argc, char **argv)
+{
+	int res;
+	int channel = 0;
+        float rxgain = 0.0;
+        float txgain = 0.0;
+
+        float old_rxgain = 0.0;
+        float old_txgain = 0.0;
+        
+        struct zt_pvt *tmp = NULL;
+
+	if (argc != 6) 
+		return RESULT_SHOWUSAGE;
+
+	if (sscanf(argv[3], "%d", &channel) != 1) 
+		return RESULT_SHOWUSAGE;
+	
+	if (sscanf(argv[4], "%f", &rxgain) != 1) 
+		return RESULT_SHOWUSAGE;
+
+	if (sscanf(argv[5], "%f", &txgain) != 1)
+		return RESULT_SHOWUSAGE;
+
+	tmp = iflist;
+	while (tmp) {
+		if (tmp->channel == channel) {
+			/* change the in memory configuration of the channel */
+			old_rxgain = tmp->rxgain;
+			old_txgain = tmp->txgain;
+			tmp->rxgain = rxgain;
+			tmp->txgain = txgain;
+			
+			/* also tweak any in-progress calls */
+			res = set_actual_gain(tmp->subs[SUB_REAL].zfd, 0, rxgain, txgain, tmp->law);
+			if (res) {
+				ast_cli(fd, "Couldn't change gains on in-progress call on channel %d\n", channel);
+				return RESULT_FAILURE;
+			} else {
+				ast_cli(fd, "Changed gains from %2.1f/%2.1f to %2.1f/%2.1f on channel %d\n", old_rxgain, old_txgain, rxgain, txgain, channel);
+				return RESULT_SUCCESS;
+			}
+		}
+		tmp = tmp->next;
+	}
+
+	ast_cli(fd, "Unable to find channel %d\n", channel);
+	return RESULT_FAILURE;
+}
+
+static struct ast_cli_entry zap_set_gain_cli =
+	{ { "zap", "set", "gain", NULL },
+	handle_zap_set_gain, "Override gain settings",
+	zap_set_gain_usage, NULL };
+
 static char show_channels_usage[] =
 	"Usage: zap show channels\n"
 	"	Shows a list of available channels\n";
@@ -8871,12 +8932,14 @@
 	ast_cli_unregister(&cli_show_channel);
 	ast_cli_unregister(&cli_destroy_channel);
 	ast_cli_unregister(&zap_show_cadences_cli);
+	ast_cli_unregister(&zap_set_gain_cli);
 	ast_manager_unregister( "ZapDialOffhook" );
 	ast_manager_unregister( "ZapHangup" );
 	ast_manager_unregister( "ZapTransfer" );
 	ast_manager_unregister( "ZapDNDoff" );
 	ast_manager_unregister( "ZapDNDon" );
 	ast_manager_unregister("ZapShowChannels");
+	ast_manager_unregister("ZapSetGain");
 	ast_unregister_application(app_callingpres);
 	ast_channel_unregister(typecompat);
 	ast_channel_unregister(type);
@@ -9672,6 +9735,7 @@
 	ast_cli_register(&cli_show_channel);
 	ast_cli_register(&cli_destroy_channel);
 	ast_cli_register(&zap_show_cadences_cli);
+	ast_cli_register(&zap_set_gain_cli);
 
 	ast_register_application(app_callingpres, change_callingpres, synopsis_callingpres, descrip_callingpres);
 	memset(round_robin, 0, sizeof(round_robin));
